/*  exmplayer, GUI front-end for mplayer.
    Copyright (C) 2010-2015 Rupesh Sreeraman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef RPHMPFEHELP_H
#define RPHMPFEHELP_H
#include <QtCore>
class rphMPFEhelp
{
public:
    rphMPFEhelp();
    QString playerhlpstrEx;
    QString playerhlpstr;
    QString audiohlpstr;
    QString videohlpstr;
    QString subtitlehlpstr;
    QString internethlpstr;
    QString keyboardhlpstr;
    QString mousehlpstr;
    QString helpcutterstr;
private:
    QString addTitle(QString str) {return QString("<p><h1><center><SPAN STYLE=\"color: blue\">"+str+"</SPAN></center></h1></p>");}
    QString addHeading(QString str){return QString("<p><h3>"+str+"</h3></p>");}
    QString addLine(QString str){return QString("<p>"+str+"</p>");}
    QString addWarningLine(QString str){return QString("<p><h4><SPAN STYLE=\"color: red\">"+str+"</SPAN></h4></p>");}
    QString addImage(QString str){return QString("<p><center><img src="+str+"</img></center></p>");}

};

#endif // RPHMPFEHELP_H
