#include <winver.h>
#include <WinVersion.h>

VS_VERSION_INFO     VERSIONINFO
FILEVERSION     VER_FILEVERSION
PRODUCTVERSION  VER_PRODUCTVERSION
FILEOS              VOS_NT_WINDOWS32
FILETYPE            VFT_APP
{
  BLOCK "StringFileInfo"
  {
    BLOCK "000004b0"
    {
            VALUE "CompanyName",        VER_COMPANYNAME_STR
            VALUE "FileDescription",    VER_FILEDESCRIPTION_STR
            VALUE "FileVersion",        VER_FILEVERSION_STR
            VALUE "InternalName",       VER_INTERNALNAME_STR
            VALUE "LegalCopyright",     VER_LEGALCOPYRIGHT_STR
            VALUE "OriginalFilename",   VER_ORIGINALFILENAME_STR
            VALUE "ProductName",        VER_PRODUCTNAME_STR
            VALUE "ProductVersion",     VER_PRODUCTVERSION_STR
    }
  }
  BLOCK "VarFileInfo"
  {
    VALUE "Translation", 0, 0x04b0
  }
}


IDI_ICON1 ICON  DISCARDABLE  "ExMplayer.ico"