/*  exmplayer, GUI front-end for mplayer.
    Copyright (C) 2010-2015 Rupesh Sreeraman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef WINVERSION_H
#define WINVERSION_H

#define VER_FILEVERSION             5,0,0,0
#define VER_FILEVERSION_STR         "5.0.0.0\0"

#define VER_PRODUCTVERSION          5,0,0,0
#define VER_PRODUCTVERSION_STR      "5.0.0.0\0"

#define VER_COMPANYNAME_STR         "Rupesh Sreeraman\0"
#define VER_FILEDESCRIPTION_STR     "ExMplayer 5.0 \0"
#define VER_INTERNALNAME_STR        "exmplayer\0"
#define VER_LEGALCOPYRIGHT_STR      "Copyright (c)2010-2015 Rupesh Sreeraman\0"
#define VER_ORIGINALFILENAME_STR    "exmplayer.exe\0"
#define VER_PRODUCTNAME_STR         "ExMplayer\0"
#define VER_COMPANYDOMAIN_STR       "http://exmplayer.sourceforge.net"

#endif // WINVERSION_H
