/*  exmplayer, GUI front-end for mplayer.
    Copyright (C) 2010-2015 Rupesh Sreeraman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef EQUALIZERTYPES_H
#define EQUALIZERTYPES_H

class EqualizerTypes
{public:
    EqualizerTypes();

    int Classical[10] ;
    int Club[10] ;
    int Dance[10] ;
    int Flat[10] ;
    int Fullbass[10] ;
    int FullbassTreble[10] ;
    int Fulltreble[10] ;
    int Headphones[10] ;
    int LargeHall[10] ;
    int Live[10] ;
    int Party[10] ;
    int Pop[10] ;
    int Reggae[10]  ;
    int Rock[10] ;
    int Ska[10] ;
    int Soft[10] ;
    int SoftRock[10]  ;
    int Techno[10] ;

};

#endif // EQUALIZERTYPES_H
