/*  exmplayer, GUI front-end for mplayer.
    Copyright (C) 2010-2015 Rupesh Sreeraman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MIXDIALOG_H
#define MIXDIALOG_H

#include <QDialog>
#include <QtCore>
#include <QFileInfo>
#include <QProcess>
#include <QMessageBox>
namespace Ui {
    class mixDialog;
}

class mixDialog : public QDialog {
    Q_OBJECT
public:
    mixDialog(QWidget *parent = 0);
    ~mixDialog();
    QTimer *coreTimer;
    QPointer<QProcess> ffProcess;
    QFileInfo *cfile;
signals:
  void showfile(QString opt,QString fn);
public slots:
  void completed(int ec);
  void updateStatus();
  void mix(QString cfn,QString af,QString  dir);
  void readffmpegoutput();
protected:
    void changeEvent(QEvent *e);
    void closeEvent(QCloseEvent * e );

private:
    Ui::mixDialog *ui;

    QString filepath;


private slots:
    void on_pushButtonof_clicked();
    void on_pushButton_clicked();
};

#endif // MIXDIALOG_H
