/*  exmplayer, GUI front-end for mplayer.
    Copyright (C) 2010-2015 Rupesh Sreeraman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef UPDATECHECK_H
#define UPDATECHECK_H

#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QXmlStreamReader>
#include <QDebug>
#include <QMap>
#include <QPair>
#include <QStringList>
#include <version.h>

class UpdateCheck:public QObject
{
    Q_OBJECT
public:
    explicit UpdateCheck(QObject *parent = 0);
    void Check();
    inline QString GetLatestStableVersion(){ return _stableVersion;}

public slots:
    void fileIsReady(QNetworkReply* reply);

signals:
    void gotUpdateChkResponse(bool updAvail,QString ver,QString relNotes);
    void NetworkError(QString);
private:
    QNetworkAccessManager * manager ;
    QString _stableVersion;
    QMap<QString,QStringList> mapVersions;

    bool isUpdated(QString currentVersion,QString myVersion);
};

#endif // UPDATECHECK_H
